!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!      FINDLAY     !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!      Code for determining biodegradation of the bottom based on          ! 
!                         Findlay-Watling model                            !
!--------------------------------------------------------------------------! 
!22/02/2014=>VB:modifications
!24/11/2005=>AD:impact area output
!---------vs1.0.1
!24/11/2005=>AD:restyling, input in paramfindlay.fi
!28/06/2004=>Vs 1.0 (C) Andrea M. Doglioli and Paolo Vassallo

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!                          SOFTWARE LICENSING                  
!    This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License (GPL) as published 
! by the Free Software Foundation, either Version 2 of the license, or 
! (at your option) any later version.
!   This program is distributed  in the hope that  it  will be useful, but 
! without any warranty; without even the implied warranty of merchantabil-
! ity or fitness for a particular purpose. See  the GPL  for more details. 
!   A copy of the GNU General Public License is available at 
!                   http://www.gnu.org/copyleft/gpl.html   
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      program FINDLAY
!--------------------------------------------------------------------------!
     
      CHARACTER*15 dataini
      character*3  recNUMsim
      character*8  fileIN
      character*8  fileOUT
      REAL, allocatable :: Vbot(:,:)
      REAL, allocatable :: Vb(:,:)
      REAL, allocatable :: FLUXB(:,:)
      REAL, allocatable :: rI   (:,:)
      REAL, allocatable :: CONCB(:,:)
      DATA SMALL/1.e-09/,UNDEF/-9999.9/
!--------------------------------------------------------------------------!
      include 'paramfindlay.fi'
      deltaXF=(deltaXLAMP/IMF)*IMLAMP   !spatial step for X [m]
      deltaYF=(deltaYLAMP/JMF)*JMLAMP   !           "          Y [m]
      rIOLAkm=IOLAMP*deltaXLAMP/1000
      rJOLAkm=JOLAMP*deltaYLAMP/1000
      delkmXF=deltaXF/1000
      delkmYF=deltaYF/1000
      IEND=(IEND/I2ORE)*I2ORE
!--------------------------------------------------------------------------!
      ALLOCATE(Vbot (IMLAMP,JMLAMP))
      ALLOCATE(Vb   (IMF,JMF))
      ALLOCATE(FLUXB(IMF,JMF))
      ALLOCATE(rI   (IMF,JMF))
      ALLOCATE(CONCB(IMF,JMF))
      
      AREA=0
      do j=1,JMF 
       do i=1,IMF
        FLUXB(i,j)=0
        CONCB(i,j)=0
       enddo
      enddo
!---------------------------------------------------
!-----output file #1=>binary input for GRADS
      open(99,file='../POM_GRADS/findlay'//recNUMsim//'.bin') !,form='unformatted') &,recordType='stream',status='new')
!-----output file #3=>ascii IMPACT AREA
      open(97,file='../POM_GRADS/findlayAREA'//recNUMsim//'.txt'
     &       ,status='new')
!-----input file from POMLAMP
      open(11,file='../POM_DATA/findlay'//recNUMsim//'.txt'
     &       ,status='old')
!#######################
      do it=1,IEND,I2ORE
       read(11,'(6X,I12)') IINT
       write(*,*) 'IINT',IINT
       read(11,*)
       read(11,*)
       do
        read(11,'(I12,2F12.4)',err=99999) Ip,Xpart,Ypart
         Ixx=INT(Xpart/deltaXF)
         Iyy=INT(Ypart/deltaYF)
         FLUXB(Ixx,Iyy)=FLUXB(Ixx,Iyy)+1
       enddo
99999  continue	
       read(11,*)
       do i=1,IMLAMP
        do j=1,JMLAMP
         read(11,'(24X,F12.4)') Vbot(i,j)
        enddo
       enddo
      
      do i=1,IMF
       do j=1,JMF
        if(IMLAMP.ne.IMF.and.JMLAMP.ne.JMF)then
         iLAMP=INT(i*IMLAMP/(IMF+0.0001))+1
         jLAMP=INT(j*JMLAMP/(JMF+0.0001))+1
        else
         iLAMP=i
         jLAMP=j
        endif

!-----bottom velocity and Oxygen Supply
        if(iLAMP.lt.IMLAMP.and.jLAMP.lt.JMLAMP)then
         Vb(i,j)=0.25*(Vbot(iLAMP,jLAMP)+Vbot(iLAMP+1,jLAMP+1)
     &                +Vbot(iLAMP,jLAMP+1)+Vbot(iLAMP+1,jLAMP))
        else
         Vb(i,j)=Vbot(iLAMP,JLAMP)
        endif

        O2sup = 736.3 + 672.5*LOG10(Vb(i,j)+SMALL)
	if(O2sup.le.0) O2sup=SMALL

!-----flux toward the bottom and Oxygen Demand
        FLUXB(i,j)=FLUXB(i,j)/(deltaXF*deltaYF) ![part/m2]
        FLUXB(i,j)=FLUXB(i,j)*12*weight         ![mmol/m2/day]        

        O2dem = 1.07*FLUXB(i,j) - 32.6
	if(O2dem.le.0) O2dem=SMALL

!-----Findlay&Watling index
        rI(i,j)= O2sup/O2dem
        if(O2sup.eq.SMALL.and.O2dem.eq.SMALL) rI(i,j)=1+2*DeltaFW

       enddo
      enddo

!-----net flux
      do j=1,JMF
       do i=1,IMF
        if(rI(i,j).lt.(1-DeltaFW)) then
         FLUXB(i,j) = FLUXB(i,j)-r_iper
        endif
        if(rI(i,j).gt.(1+DeltaFW)) then
         FLUXB(i,j) = FLUXB(i,j)-r_olig
	endif
        if((rI(i,j).ge.(1-DeltaFW)).and.(rI(i,j).le.(1+DeltaFW)) ) then
         FLUXB(i,j) = FLUXB(i,j)-r_meso
        endif
       enddo
      enddo

!-----bottom concentration
      do j=1,JMF
       do i=1,JMF
        CONCB(i,j)=CONCB(i,j)+FLUXB(i,j)/12 ![mmol/m2]
	if(CONCB(i,j).lt.0) CONCB(i,j)=0.
!-----impact area
        if (CONCB(i,j) .gt. CONCBcrit) then
         AREA=AREA+deltaXF*deltaYF ![m]
        endif
       enddo
      enddo
      write(97,*) IT,AREA
      AREA=0

!****************** Writting BIN file for GRADS **********************
	do j=JMF,1,-1          
         do i=1,IMF            
           write(99) Vb(i,j)
         enddo
        enddo
	do j=JMF,1,-1          
         do i=1,IMF            
           write(99) rI(i,j)
         enddo
        enddo
	do j=JMF,1,-1          
         do i=1,IMF            
           write(99) FLUXB(i,j)
         enddo
        enddo
	do j=JMF,1,-1          
         do i=1,IMF            
           write(99) CONCB(i,j)
         enddo
        enddo
!-----Reset Vbot and FLUX 
      do j=1,JMF
       do i=1,IMF
        FLUXB(i,j)=0
       enddo
      enddo

      enddo!it
!#######################

      close(99)
      close(97)
      close(11)
      
!-----output file #2=>ascii input for GRADS
      open(98,file='../POM_GRADS/findlay'//recNUMsim//'.ctl'
     &      ,status='new')

11109  format(A6,F8.2)
11110  format(50A)
11111  format(1A,'DEF ',I3,' LINEAR ',A4,A8)
11112  format(1A,'DEF ',I4,' LINEAR ',A15,I15,'mn') 

11113  format(1A,'DEF ',I3,' LINEAR ',F12.5,F12.5)

11114  format(A5,I3,37A)
11115  format(1A,'DEF ',I3,' LEVELS ',12I6)


      write(98,11110)'DSET ./findlay'//recNUMsim//'.bin'

      write(98,11110)'TITLE ------- SIMULAZIONE '//recNUMsim//' -------'
      write(98,11109)'UNDEF ',undef
      write(98,11110)'OPTIONS yrev'

      mnPRTD1=120!NINT(PRTD1*24*60)  
      itMAX=IEND/I2ORE
      scala=1.*IMLAMP/IMF

      write(98,11113) 'X',IMF,rIOLAkm,delkmXF
      write(98,11113) 'Y',JMF,rJOLAkm,delkmYF
      write(98,11110) 'ZDEF 1 levels 1'

      write(98,11112) 'T',iTMAX,dataini,mnPRTD1


      write(98,11110) 'VARS 4'
      write(98,11110) 'VB     0  99 [cm/s]'     
      write(98,11110) 'rI     0  99 [adim]'     
      write(98,11110) 'FLUXB  0  99 flusso netto [mmol/m2/d]'     
      write(98,11110) 'CONCB  0  99 concentrazione fondo [mmol/m2]'     

      write(98,11110) 'ENDVARS'

      write(98,*) 
      write(98,11110) ' Parametri utilizzati per Findlay'
      write(98,*) ' deltaFW',                   DeltaFW
      write(98,*) ' r_oligo [mmol/m2/day]',     r_olig
      write(98,*) ' r_iper [mmol/m2/day]',      r_iper
      write(98,*) ' r_meso [mmol/m2/day]',      r_meso
      write(98,*) ' peso particella [mol]',     weight


      close(98)

      STOP
      END 